# Generated by roxygen2: do not edit by hand

export(download_cdl_mat_files)
export(get_mat_data)
export(get_trans_mat)
export(simulate_raster_patch)
import(dplyr)
import(landscapemetrics)
import(readxl)
importFrom(Rcpp,sourceCpp)
importFrom(dplyr,intersect)
importFrom(dplyr,union)
importFrom(stats,na.omit)
importFrom(utils,download.file)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(utils,unzip)
useDynLib(cdlsim, .registration = TRUE)
